@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,7 +21,8 @@ Means to create XML elements from Nuitka tree nodes and to convert the
 XML tree to ASCII or output it.
 """
 
-from nuitka.utils import Utils
+from nuitka.__past__ import StringIO
+from nuitka.PythonVersions import python_version
 
 from . import Tracing
 
@@ -65,20 +66,34 @@ except ImportError:
 
         Element = xml.etree.ElementTree.Element
         xml_tostring = lambda tree: xml_module.tostring(indent(tree))
-
     except ImportError:
         xml_module = None
         Element = None
         xml_tostring = None
 
+# TODO: Use the writer to create the XML we output. That should be more
+# scalable and/or faster.
+try:
+    import lxml.xmlfile
+
+    xml_writer = lxml.xmlfile
+except ImportError:
+    xml_writer = None
+
+
 def toString(tree):
     result = xml_tostring(tree)
 
-    if Utils.python_version >= 300:
+    if python_version >= 300:
         result = result.decode("utf-8")
 
     return result
 
+
+def fromString(text):
+    return xml_module.parse(StringIO(text)).getroot()
+
+
 def dump(tree):
     value = toString(tree).rstrip()
 
