@@ -30,6 +30,7 @@ for a single module only.
 import ctypes
 import re
 import struct
+import sys
 from logging import warning
 
 import marshal
@@ -59,6 +60,47 @@ def generateConstantReferenceCode(to_name, expression, emit, context):
         context  = context
     )
 
+
+def generateConstantNoneReferenceCode(to_name, expression, emit, context):
+    """ Assign 'None' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=W0613
+
+    emit(
+        "%s = Py_None;" % to_name
+    )
+
+
+def generateConstantTrueReferenceCode(to_name, expression, emit, context):
+    """ Assign 'True' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=W0613
+
+    emit(
+        "%s = Py_True;" % to_name
+    )
+
+
+def generateConstantFalseReferenceCode(to_name, expression, emit, context):
+    """ Assign 'False' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=W0613
+
+    emit(
+        "%s = Py_False;" % to_name
+    )
+
+
+def generateConstantEllipsisReferenceCode(to_name, expression, emit, context):
+    """ Assign 'Ellipsis' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=W0613
+
+    emit(
+        "%s = Py_Ellipsis;" % to_name
+    )
+
+
 # One global stream of constant information. In the future it might make
 # sense to have per module ones, for better locality of indexes within it,
 # but we don't do this yet.
@@ -300,14 +342,6 @@ assert( hash_%(constant_identifier)s == DEEP_HASH( %(constant_identifier)s ) );"
              }
         )
 
-        if Options.isExperimental():
-            check(
-                """\
-if ( hash_%(constant_identifier)s == -1 ) puts("Note: Weak hash for: %(constant_identifier)s.");""" % {
-                "constant_identifier" : constant_identifier
-                }
-            )
-
 
 def __addConstantInitCode(context, emit, check, constant_type, constant_value,
                           constant_identifier, module_level):
@@ -870,8 +904,7 @@ def getConstantAccess(to_name, constant, emit, context):
 
             ref_count = 0
     else:
-        code = getConstantCode(
-            context  = context,
+        code = context.getConstantCode(
             constant = constant
         )
 
@@ -1001,8 +1034,14 @@ def getConstantsDefinitionCode(context):
         context = context
     )
 
+    if Options.shallMakeModule():
+        sys_executable = None
+    else:
+        sys_executable = context.getConstantCode(sys.executable)
+
     return template_constants_reading % {
         "constant_declarations" : '\n'.join(constant_declarations),
         "constant_inits"        : indented(constant_inits),
-        "constant_checks"       : indented(constant_checks)
+        "constant_checks"       : indented(constant_checks),
+        "sys_executable"        : sys_executable
     }
