@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,9 +32,9 @@ from nuitka.Builtins import (
     builtin_names
 )
 from nuitka.optimizations import BuiltinOptimization
-from nuitka.utils.Utils import python_version
+from nuitka.PythonVersions import python_version
 
-from .ConstantRefNodes import ExpressionConstantRef
+from .ConstantRefNodes import makeConstantRefNode
 from .ExceptionNodes import ExpressionBuiltinMakeException
 from .NodeBases import CompileTimeConstantExpressionMixin, NodeBase
 
@@ -64,7 +64,7 @@ class ExpressionBuiltinRefBase(CompileTimeConstantExpressionMixin, NodeBase):
         return False
 
     def getStrValue(self):
-        return ExpressionConstantRef(
+        return makeConstantRefNode(
             constant      = str(self.getCompileTimeConstant()),
             user_provided = True,
             source_ref    = self.getSourceReference(),
@@ -99,7 +99,7 @@ class ExpressionBuiltinRef(ExpressionBuiltinRefBase):
         }
 
         if self.builtin_name in quick_names:
-            new_node = ExpressionConstantRef(
+            new_node = makeConstantRefNode(
                 constant   = quick_names[self.builtin_name],
                 source_ref = self.getSourceReference()
             )
