@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,7 +26,8 @@ them.
 
 """
 
-from nuitka.utils.Utils import python_version
+from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
+from nuitka.PythonVersions import python_version
 
 from .StatementNodes import StatementsSequence
 
@@ -81,12 +82,35 @@ class StatementsFrame(StatementsSequence):
         return result
 
     def getDetailsForDisplay(self):
-        result = self.getDetails()
+        result = {
+            "guard_mode"  : self.guard_mode,
+        }
+
+        result.update(StatementsSequence.getDetails(self))
 
         result.update(self.code_object.getDetails())
 
         return result
 
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        code_object_args = {}
+        other_args = {}
+
+        for key, value in args.items():
+            if key.startswith("co_"):
+                code_object_args[key] = value
+            else:
+                other_args[key] = value
+
+        code_object = CodeObjectSpec(**code_object_args)
+
+        return cls(
+            code_object = code_object,
+            source_ref  = source_ref,
+            **other_args
+        )
+
     def getGuardMode(self):
         return self.guard_mode
 
@@ -107,9 +131,14 @@ class StatementsFrame(StatementsSequence):
         """
         provider = self.getParentVariableProvider()
 
-        # TODO: Bad for in-lining of these.
         if provider.isExpressionFunctionBody():
-            self.code_object.updateLocalNames(provider.getLocalVariableNames())
+            self.code_object.updateLocalNames(
+                [
+                    variable.getName() for
+                    variable in
+                    provider.getLocalVariables()
+                ]
+            )
 
     def markAsFrameExceptionPreserving(self):
         self.needs_frame_exception_preserve = True
