@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,10 +29,12 @@ Coroutines and generators live in their dedicated module and share base
 classes.
 """
 
-from nuitka import Options, VariableRegistry, Variables
+from nuitka import Options, Variables
+from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
+from nuitka.nodes.FutureSpecs import fromFlags
 from nuitka.optimizations.FunctionInlining import convertFunctionCallToOutline
+from nuitka.PythonVersions import python_version
 from nuitka.tree.Extractions import updateVariableUsage
-from nuitka.utils import Utils
 
 from .Checkers import checkStatementsSequenceOrNone
 from .IndicatorMixins import (
@@ -54,13 +56,14 @@ from .NodeMakingHelpers import (
     makeRaiseExceptionReplacementExpressionFromInstance,
     wrapExpressionWithSideEffects
 )
-from .ParameterSpecs import TooManyArguments, matchCall
+from .ParameterSpecs import ParameterSpec, TooManyArguments, matchCall
 
 
 class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
                                  ClosureGiverNodeBase, ExpressionMixin):
 
-    def __init__(self, provider, name, code_prefix, is_class, source_ref):
+    def __init__(self, provider, name, code_prefix, is_class, flags, source_ref,
+                 body = None):
         ClosureTakerMixin.__init__(
             self,
             provider      = provider,
@@ -77,26 +80,26 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
         ChildrenHavingMixin.__init__(
             self,
             values = {
-                "body" : None # delayed
+                "body" : body # Might be None initially in some cases.
             }
         )
 
+        # Special things, "has_super" indicates presence of "super" in variable
+        # usage, which modifies some behaviors.
+        self.flags = flags
+
         # Hack: This allows some APIs to work although this is not yet
         # officially a child yet. Important during building.
         self.parent = provider
 
         # Python3.4: Might be overridden by global statement on the class name.
         # TODO: Make this class only code.
-        if Utils.python_version >= 340:
+        if python_version >= 340:
             self.qualname_provider = provider
 
         # Non-local declarations.
         self.non_local_declarations = []
 
-        # Indicator, if this is a function that uses "super", because if it
-        # does, it would like to get the final "__class__" attached.
-        self.has_super = False
-
         # Register ourselves immediately with the module.
         provider.getParentModule().addFunction(self)
 
@@ -117,23 +120,23 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
 
         return None
 
-    def isClassClosureTaker(self):
-        return self.has_super
+    def hasFlag(self, flag):
+        return flag in self.flags
 
     def getLocalsMode(self):
-        if Utils.python_version >= 300:
+        if python_version >= 300:
             return "updated"
         elif self.isEarlyClosure() or self.isUnoptimized():
             return "updated"
         else:
             return "copy"
 
+    def hasVariableName(self, variable_name):
+        return variable_name in self.providing or variable_name in self.temp_variables
+
     def getVariables(self):
         return self.providing.values()
 
-    def getLocalVariableNames(self):
-        return Variables.getNames(self.getLocalVariables())
-
     def getLocalVariables(self):
         return [
             variable for
@@ -172,6 +175,10 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
             owner         = self,
             variable_name = variable.getName()
         )
+        for variable_trace in variable.traces:
+            if variable_trace.getOwner() is self:
+                new_variable.addTrace(variable_trace)
+        new_variable.updateUsageState()
 
         self.providing[variable.getName()] = new_variable
 
@@ -181,8 +188,6 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
             new_variable = new_variable
         )
 
-        VariableRegistry.addVariableUsage(new_variable, self)
-
     def removeUserVariable(self, variable):
         assert variable in self.providing.values(), (self.providing, variable)
 
@@ -219,7 +224,7 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
             # For "exec" containing/star import containing, we get a
             # closure variable already, but if it is a module variable,
             # only then make it a maybe local variable.
-            if self.isUnoptimized() and result.isModuleVariable():
+            if not self.isExpressionClassBody() and self.isUnoptimized() and result.isModuleVariable():
                 result = Variables.MaybeLocalVariable(
                     owner          = self,
                     maybe_variable = result
@@ -253,6 +258,30 @@ class ExpressionFunctionBodyBase(ClosureTakerMixin, ChildrenHavingMixin,
     def getNonlocalDeclarations(self):
         return self.non_local_declarations
 
+    def getFunctionName(self):
+        return self.name
+
+    def getFunctionQualname(self):
+        """ Function __qualname__ new in CPython3.3
+
+        Should contain some kind of full name descriptions for the closure to
+        recognize and will be used for outputs.
+        """
+
+        function_name = self.getFunctionName()
+
+        if python_version < 340:
+            provider = self.getParentVariableProvider()
+        else:
+            provider = self.qualname_provider
+
+        if provider.isCompiledPythonModule():
+            return function_name
+        elif provider.isExpressionClassBody():
+            return provider.getFunctionQualname() + '.' + function_name
+        else:
+            return provider.getFunctionQualname() + ".<locals>." + function_name
+
     def computeExpression(self, constraint_collection):
         assert False
 
@@ -285,27 +314,25 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
         "body" : checkStatementsSequenceOrNone
     }
 
-    if Utils.python_version >= 340:
+    if python_version >= 340:
         qualname_setup = None
 
-    def __init__(self, provider, name, doc, parameters, is_class, source_ref):
+    def __init__(self, provider, name, doc, parameters, flags, source_ref,
+                 body = None):
         while provider.isExpressionOutlineBody():
             provider = provider.getParentVariableProvider()
 
-        if is_class:
-            code_prefix = "class"
-        else:
-            code_prefix = "function"
-
         if name == "<listcontraction>":
-            assert Utils.python_version >= 300
+            assert python_version >= 300
 
         ExpressionFunctionBodyBase.__init__(
             self,
             provider    = provider,
             name        = name,
-            code_prefix = code_prefix,
-            is_class    = is_class,
+            code_prefix = "function",
+            is_class    = False,
+            flags       = flags,
+            body        = body,
             source_ref  = source_ref
         )
 
@@ -313,8 +340,6 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
 
         MarkUnoptimizedFunctionIndicator.__init__(self)
 
-        self.is_class = is_class
-
         self.doc = doc
 
         # Indicator if the return value exception might be required.
@@ -332,9 +357,8 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
         self.parameters = parameters
         self.parameters.setOwner(self)
 
-        self.registerProvidedVariables(
-            *self.parameters.getVariables()
-        )
+        for variable in self.parameters.getAllVariables():
+            self.registerProvidedVariable(variable)
 
     def getDetails(self):
         return {
@@ -345,36 +369,57 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
             "doc"        : self.doc
         }
 
+    def getDetailsForDisplay(self):
+        result = {
+            "name"       : self.getFunctionName(),
+            "provider"   : self.provider.getCodeName(),
+            "flags"      : self.flags
+        }
+
+        result.update(self.parameters.getDetails())
+
+        result["code_flags"] = ','.join(self.getSourceReference().getFutureSpec().asFlags())
+
+        if self.doc is not None:
+            result["doc"] = self.doc
+
+        return result
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        assert provider is not None
+
+        parameter_spec_args = {}
+        other_args = {}
+
+        for key, value in args.items():
+            if key.startswith("ps_"):
+                parameter_spec_args[key] = value
+            elif key == "code_flags":
+                source_ref.future_spec = fromFlags(value)
+            else:
+                other_args[key] = value
+
+        parameters = ParameterSpec(**parameter_spec_args)
+
+        # The empty doc string and no doc string are distinguished by presence. The
+        # most common case is going to be not present.
+        if "doc" not in other_args:
+            other_args["doc"] = None
+
+        return cls(
+            provider   = provider,
+            parameters = parameters,
+            source_ref = source_ref,
+            **other_args
+        )
+
     def getDetail(self):
         return "named %s with %s" % (self.getFunctionName(), self.parameters)
 
     def getParent(self):
         assert False
 
-    def getFunctionName(self):
-        return self.name
-
-    def getFunctionQualname(self):
-        """ Function __qualname__ new in CPython3.3
-
-        Should contain some kind of full name descriptions for the closure to
-        recognize and will be used for outputs.
-        """
-
-        function_name = self.getFunctionName()
-
-        if Utils.python_version < 340:
-            provider = self.getParentVariableProvider()
-        else:
-            provider = self.qualname_provider
-
-        if provider.isCompiledPythonModule():
-            return function_name
-        elif provider.isExpressionClassBody():
-            return provider.getFunctionQualname() + '.' + function_name
-        else:
-            return provider.getFunctionQualname() + ".<locals>." + function_name
-
     def getDoc(self):
         return self.doc
 
@@ -428,56 +473,6 @@ class ExpressionFunctionBody(ExpressionFunctionBodyBase,
     def needsExceptionReturnValue(self):
         return self.return_exception
 
-
-
-
-class ExpressionClassBody(ExpressionFunctionBody):
-    # pylint: disable=R0901
-    kind = "EXPRESSION_CLASS_BODY"
-
-    named_children = (
-        "body",
-    )
-
-    checkers = {
-        # TODO: Is "None" really an allowed value.
-        "body" : checkStatementsSequenceOrNone
-    }
-
-    # TODO: Everybody should check base instead.
-    @staticmethod
-    def isExpressionFunctionBody():
-        return True
-
-    def getVariableForClosure(self, variable_name):
-        # print( "getVariableForClosure", self, variable_name )
-
-        # The class bodies provide no closure, except under CPython3.x, there
-        # they provide "__class__" but nothing else.
-
-        if variable_name == "__class__":
-            if Utils.python_version < 300:
-                return self.provider.getVariableForClosure(
-                    "__class__"
-                )
-            elif Utils.python_version >= 340 and False: # TODO: Temporarily reverted
-                result = self.getTempVariable(
-                    temp_scope = None,
-                    name       = "__class__"
-                )
-
-                return result
-            else:
-                return ExpressionFunctionBody.getVariableForClosure(
-                    self,
-                    variable_name = "__class__"
-                )
-        else:
-            return self.provider.getVariableForClosure(
-                variable_name
-            )
-
-
 def convertNoneConstantOrEmptyDictToNone(node):
     if node is None:
         return None
@@ -500,7 +495,7 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
     # all kinds go last.
 
     # A bug of CPython3.x not fixed before version 3.4, see bugs.python.org/issue16967
-    kw_defaults_before_defaults = Utils.python_version < 340
+    kw_defaults_before_defaults = python_version < 340
 
     if kw_defaults_before_defaults:
         named_children = (
@@ -545,6 +540,34 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
             "code_object" : self.code_object
         }
 
+    def getDetailsForDisplay(self):
+        if self.code_object:
+            return self.code_object.getDetails()
+        else:
+            return {}
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        code_object_specs = {}
+        other_args = {}
+
+        for key, value in args.items():
+            if key.startswith("co_"):
+                code_object_specs[key] = value
+            else:
+                other_args[key] = value
+
+        if code_object_specs:
+            code_object = CodeObjectSpec(**code_object_specs)
+        else:
+            code_object = None
+
+        return cls(
+            code_object = code_object,
+            source_ref  = source_ref,
+            **other_args
+        )
+
     def computeExpression(self, constraint_collection):
         defaults = self.getDefaults()
 
@@ -659,7 +682,7 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
             values = [
                 args_dict[name]
                 for name in
-                call_spec.getAllNames()
+                call_spec.getParameterNames()
             ]
 
             result = ExpressionFunctionCall(
@@ -730,8 +753,9 @@ error""" % self.getName()
 class ExpressionFunctionRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_FUNCTION_REF"
 
-    def __init__(self, function_body, source_ref):
-        assert function_body.isExpressionFunctionBody()
+    def __init__(self, source_ref, function_body = None, code_name = None):
+        assert function_body is not None or code_name is not None
+        assert code_name != "None"
 
         NodeBase.__init__(
             self,
@@ -739,6 +763,7 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
         )
 
         self.function_body = function_body
+        self.code_name = code_name
 
     def getName(self):
         return self.function_body.getName()
@@ -750,10 +775,18 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
 
     def getDetailsForDisplay(self):
         return {
-            "function" : self.function_body.getCodeName()
+            "code_name" : self.getFunctionBody().getCodeName()
         }
 
     def getFunctionBody(self):
+        if self.function_body is None:
+            module_code_name, _ = self.code_name.split("$$$", 1)
+
+            from nuitka.ModuleRegistry import getModuleFromCodeName
+            module = getModuleFromCodeName(module_code_name)
+
+            self.function_body = module.getFunctionFromCodeName(self.code_name)
+
         return self.function_body
 
     def computeExpressionRaw(self, constraint_collection):
@@ -890,4 +923,8 @@ class ExpressionFunctionQualnameRef(CompileTimeConstantExpressionMixin,
             constant = self.function_body.getFunctionQualname()
         )
 
-        return result, "new_constant", "Delayed __qualname__ resolution."
+        return (
+            result,
+            "new_constant",
+            "Executed '__qualname__' resolution to '%s'." % self.function_body.getFunctionQualname()
+        )
