@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,7 @@ whose implementation lives here. The creation itself also lives here.
 
 """
 
-from nuitka.utils import Utils
+from nuitka.PythonVersions import python_version
 
 from .Checkers import checkStatementsSequenceOrNone
 from .FunctionNodes import ExpressionFunctionBodyBase
@@ -91,16 +91,17 @@ class ExpressionGeneratorObjectBody(ExpressionFunctionBodyBase,
         "body" : checkStatementsSequenceOrNone
     }
 
-    if Utils.python_version >= 340:
+    if python_version >= 340:
         qualname_setup = None
 
-    def __init__(self, provider, name, source_ref):
+    def __init__(self, provider, name, flags, source_ref):
         ExpressionFunctionBodyBase.__init__(
             self,
             provider    = provider,
             name        = name,
             is_class    = False,
             code_prefix = "genexpr" if name == "<genexpr>" else "genobj",
+            flags       = flags,
             source_ref  = source_ref
         )
 
@@ -116,10 +117,6 @@ class ExpressionGeneratorObjectBody(ExpressionFunctionBodyBase,
     def getFunctionQualname(self):
         return self.getParentVariableProvider().getFunctionQualname()
 
-    @staticmethod
-    def isExpressionFunctionBody():
-        return True
-
     def markAsNeedsGeneratorReturnHandling(self, value):
         self.needs_generator_return_exit = max(
             self.needs_generator_return_exit,
