@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,6 +29,7 @@ compile time constant.
 
 from logging import warning
 
+from nuitka.__past__ import unicode  # pylint: disable=W0622
 from nuitka.importing.Importing import (
     findModule,
     getModuleNameAndKindFromFilename
@@ -37,7 +38,7 @@ from nuitka.importing.Recursion import decideRecursion, recurseTo
 from nuitka.importing.Whitelisting import getModuleWhiteList
 from nuitka.utils import Utils
 
-from .ConstantRefNodes import ExpressionConstantRef
+from .ConstantRefNodes import makeConstantRefNode
 from .NodeBases import (
     ExpressionChildrenHavingBase,
     ExpressionMixin,
@@ -54,7 +55,7 @@ class ExpressionImportModule(NodeBase, ExpressionMixin):
     _warned_about = set()
 
     def __init__(self, module_name, import_list, level, source_ref):
-        assert type(module_name) is str, type(module_name)
+        assert type(module_name) in (str, unicode), type(module_name)
 
         NodeBase.__init__(
             self,
@@ -62,13 +63,24 @@ class ExpressionImportModule(NodeBase, ExpressionMixin):
         )
 
         self.module_name = module_name
+
+        if type(import_list) is str:
+            if import_list == "":
+                import_list = ()
+            else:
+                import_list = import_list.split(',')
+
         self.import_list = import_list
-        self.level = level
 
-        self.module = None
+        self.level = int(level)
 
-        self.attempted_recurse = False
-        self.found_modules = ()
+        # Are we pointing to a known module or not. If so, we can expect it to
+        # be in the module registry.
+        self.found = None
+
+        # If we are pointing to a known module, this are modules behind the
+        # "import_list".
+        self.found_modules = None
 
     def getDetails(self):
         return {
@@ -77,6 +89,32 @@ class ExpressionImportModule(NodeBase, ExpressionMixin):
             "import_list" : self.import_list
         }
 
+    def getDetailsForDisplay(self):
+        result = {
+            "module_name" : self.module_name,
+            "level"       : self.level,
+        }
+
+        if self.import_list is not None:
+            result["import_list"] = ','.join(self.import_list)
+
+        return result
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        if "import_list" in args:
+            import_list = args["import_list"].split(',')
+            del args["import_list"]
+        else:
+            import_list = None
+
+        return cls(
+            import_list = import_list,
+            source_ref  = source_ref,
+            **args
+        )
+
+
     def getModuleName(self):
         return self.module_name
 
@@ -92,15 +130,6 @@ class ExpressionImportModule(NodeBase, ExpressionMixin):
         else:
             return self.level
 
-    def getModule(self):
-        return self.module
-
-    def setModule(self, module):
-        # Modules have no parent.
-        assert module.parent is None
-
-        self.module = module
-
     def _consider(self, constraint_collection, module_filename, module_package):
         assert module_package is None or \
               (type(module_package) is str and module_package != "")
@@ -159,7 +188,7 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                     )
 
     def _attemptRecursion(self, constraint_collection):
-        assert self.getModule() is None
+        found = False
 
         parent_module = self.getParentModule()
 
@@ -184,7 +213,8 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
             )
 
             if imported_module is not None:
-                self.setModule(imported_module)
+                found = imported_module.getFullName()
+
                 self.found_modules = []
 
                 import_list = self.getImportList()
@@ -210,25 +240,22 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                             )
 
                             if sub_imported_module is not None:
-                                self.found_modules.append(sub_imported_module)
+                                self.found_modules.append(sub_imported_module.getFullName())
 
+            return found
 
     def computeExpression(self, constraint_collection):
         # Attempt to recurse if not already done.
-        if not self.attempted_recurse:
-            self._attemptRecursion(
+        if self.found is None:
+            self.found = self._attemptRecursion(
                 constraint_collection = constraint_collection
             )
 
-            self.attempted_recurse = True
-
-        if self.getModule() is not None:
-            from nuitka.ModuleRegistry import addUsedModule
-            addUsedModule(self.getModule())
+        if self.found:
+            constraint_collection.onUsedModule(self.found)
 
             for found_module in self.found_modules:
-                addUsedModule(found_module)
-
+                constraint_collection.onUsedModule(found_module)
 
         # When a module is recursed to and included, we know it won't raise,
         # right? But even if you import, that successful import may still raise
@@ -293,7 +320,7 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
     def __init__(self, name, import_globals, import_locals, fromlist, level,
                 source_ref):
         if fromlist is None:
-            fromlist = ExpressionConstantRef(
+            fromlist = makeConstantRefNode(
                 constant   = (),
                 source_ref = source_ref
             )
@@ -301,7 +328,7 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
         if level is None:
             level = 0 if source_ref.getFutureSpec().isAbsoluteImport() else -1
 
-            level = ExpressionConstantRef(
+            level = makeConstantRefNode(
                 constant   = level,
                 source_ref = source_ref
             )
@@ -337,7 +364,7 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
            fromlist.isExpressionConstantRef() and \
            level.isExpressionConstantRef():
 
-            if module_name.isStringConstant():
+            if module_name.isStringConstant() or module_name.isUnicodeConstant():
                 new_node = ExpressionImportModule(
                     module_name = module_name.getConstant(),
                     import_list = fromlist.getConstant(),
@@ -352,18 +379,18 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
                 return (
                     new_node,
                     "new_import",
-                    "Replaced __import__ call with module import expression."
+                    "Replaced '__import__' call with module import expression."
                 )
             else:
                 # Non-strings is going to raise an error.
                 new_node, change_tags, message = constraint_collection.getCompileTimeComputationResult(
                     node        = self,
                     computation = lambda : __import__(module_name.getConstant()),
-                    description = "Replaced __import__ call with non-string module name argument."
+                    description = "Replaced '__import__' call with non-string module name argument."
                 )
 
                 # Must fail, must not go on when it doesn't.
-                assert change_tags == "new_raise"
+                assert change_tags == "new_raise", module_name
 
                 return new_node, change_tags, message
 
@@ -425,7 +452,9 @@ class ExpressionImportName(ExpressionChildrenHavingBase):
         return self.import_name
 
     def getDetails(self):
-        return { "import_name" : self.getImportName() }
+        return {
+            "import_name" : self.getImportName()
+        }
 
     def getDetail(self):
         return "import %s from %s" % (
