@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -62,16 +62,6 @@ class StatementsSequence(StatementChildrenHavingBase):
     getStatements = StatementChildrenHavingBase.childGetter("statements")
     setStatements = StatementChildrenHavingBase.childSetter("statements")
 
-    def getDetailsForDisplay(self):
-        if self.getStatements():
-            return {
-                "statement_count" : len(self.getStatements())
-            }
-        else:
-            return {
-                "statement_count" : 0
-            }
-
     # Overloading name based automatic check, so that derived ones know it too.
     def isStatementsSequence(self):
         # Virtual method, pylint: disable=R0201
