@@ -80,14 +80,17 @@ from nuitka.nodes.BuiltinTypeNodes import (
     ExpressionBuiltinTuple
 )
 from nuitka.nodes.BuiltinVarsNodes import ExpressionBuiltinVars
-from nuitka.nodes.CallNodes import ExpressionCallEmpty, ExpressionCallNoKeywords
+from nuitka.nodes.CallNodes import (
+    ExpressionCallEmpty,
+    ExpressionCallNoKeywords
+)
 from nuitka.nodes.ClassNodes import ExpressionBuiltinType3
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ExecEvalNodes import (
     ExpressionBuiltinCompile,
     ExpressionBuiltinEval
@@ -124,9 +127,8 @@ from nuitka.tree.Helpers import (
     makeStatementsSequenceFromStatement
 )
 from nuitka.tree.ReformulationExecStatements import wrapEvalGlobalsAndLocals
-from nuitka.tree. \
-    ReformulationTryFinallyStatements import makeTryFinallyStatement
-from nuitka.VariableRegistry import addVariableUsage
+from nuitka.tree.ReformulationTryFinallyStatements import \
+    makeTryFinallyStatement
 
 from . import BuiltinOptimization
 
@@ -538,7 +540,7 @@ if python_version < 300:
                                 called     = ExpressionAttributeLookup(
                                     source         = ExpressionBuiltinOpen(
                                         filename   = filename,
-                                        mode       = ExpressionConstantRef(
+                                        mode       = makeConstantRefNode(
                                             constant   = "rU",
                                             source_ref = source_ref
                                         ),
@@ -625,7 +627,7 @@ def eval_extractor(node):
             )
         )
 
-        strip_choice =  ExpressionConstantRef(
+        strip_choice =  makeConstantRefNode(
             constant   = (" \t",),
             source_ref = source_ref
         )
@@ -646,7 +648,7 @@ def eval_extractor(node):
                     ),
                     source_ref = source_ref
                 ),
-                expression_yes = ExpressionConstantRef(
+                expression_yes = makeConstantRefNode(
                     constant   = (b" \t",),
                     source_ref = source_ref
                 ),
@@ -867,8 +869,6 @@ def super_extractor(node):
             not (type_arg_owner.isExpressionFunctionBody() or \
                  type_arg_owner.isExpressionClassBody()):
                 type_arg = None
-            else:
-                addVariableUsage(type_arg.getVariable(), provider)
 
             if type_arg is None:
                 return makeRaiseExceptionReplacementExpression(
