@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Constraint collection
+""" Trace collection (also often still referred to as constraint collection).
 
 At the core of value propagation there is the collection of constraints that
 allow to propagate knowledge forward or not.
@@ -26,8 +26,14 @@ This is about collecting these constraints and to manage them.
 import contextlib
 from logging import debug
 
-from nuitka import Tracing, VariableRegistry
+from nuitka import Tracing, Variables
 from nuitka.__past__ import iterItems  # Python3 compatibility.
+from nuitka.containers.oset import OrderedSet
+from nuitka.importing.ImportCache import (
+    getImportedModuleByName,
+    isImportedModuleByName
+)
+from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
 from nuitka.PythonVersions import python_version
 
@@ -42,7 +48,6 @@ from .VariableTraces import (
 
 signalChange = None
 
-
 class VariableUsageTrackingMixin:
 
     def initVariable(self, variable):
@@ -319,7 +324,7 @@ class CollectionStartpointMixin:
         return trace
 
     def updateFromCollection(self, old_collection):
-        VariableRegistry.updateFromCollection(old_collection, self)
+        Variables.updateFromCollection(old_collection, self)
 
     @contextlib.contextmanager
     def makeAbortStackContext(self, catch_breaks, catch_continues,
@@ -369,9 +374,12 @@ class ConstraintCollectionBase(CollectionTracingMixin):
 
     @staticmethod
     def signalChange(tags, source_ref, message):
-        # This is monkey patches from another module, pylint: disable=E1102
+        # This is monkey patched from another module, pylint: disable=E1102
         signalChange(tags, source_ref, message)
 
+    def onUsedModule(self, module):
+        return self.parent.onUsedModule(module)
+
     @staticmethod
     def mustAlias(a, b):
         if a.isExpressionVariableRef() and b.isExpressionVariableRef():
@@ -381,7 +389,10 @@ class ConstraintCollectionBase(CollectionTracingMixin):
 
     @staticmethod
     def mustNotAlias(a, b):
-        # TODO: not yet really implemented, pylint: disable=W0613
+        # TODO: not yet really implemented
+        if a.isExpressionConstantDictRef() and b.isExpressionConstantRef():
+            return True
+
         return False
 
     def removeKnowledge(self, node):
@@ -397,7 +408,7 @@ class ConstraintCollectionBase(CollectionTracingMixin):
 
                 self.markActiveVariableAsUnknown(variable)
 
-            elif python_version >= 300 or variable.isSharedTechnically():
+            elif python_version >= 300 or variable.isSharedTechnically() is not False:
                 # print variable
 
                 # TODO: Could be limited to shared variables that are actually
@@ -778,3 +789,15 @@ class ConstraintCollectionModule(CollectionStartpointMixin,
             name   = "module",
             parent = None
         )
+
+        self.used_modules = OrderedSet()
+
+    def onUsedModule(self, module_name):
+        self.used_modules.add(module_name)
+
+        if isImportedModuleByName(module_name):
+            module = getImportedModuleByName(module_name)
+            addUsedModule(module)
+
+    def getUsedModules(self):
+        return self.used_modules
