@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -38,12 +38,12 @@ from nuitka.nodes.BuiltinIteratorNodes import (
     ExpressionBuiltinIter1,
     ExpressionBuiltinNext1
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
-from nuitka.nodes.ContainerOperationNodes import ExpressionDictOperationUpdate
 from nuitka.nodes.DictionaryNodes import (
     ExpressionKeyValuePair,
-    ExpressionMakeDict
+    ExpressionMakeDict,
+    StatementDictOperationUpdate
 )
 from nuitka.nodes.ExceptionNodes import (
     ExpressionBuiltinMakeException,
@@ -55,11 +55,10 @@ from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionCreation,
     ExpressionFunctionRef
 )
-from nuitka.nodes.LoopNodes import StatementBreakLoop, StatementLoop
+from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.OperatorNodes import ExpressionOperationBinary
 from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.StatementNodes import StatementExpressionOnly
 from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
@@ -96,7 +95,6 @@ def buildDictionaryNode(provider, node, source_ref):
     return makeDictCreationOrConstant(
         keys       = buildNodeList(provider, node.keys, source_ref),
         values     = buildNodeList(provider, node.values, source_ref),
-        lazy_order = False,
         source_ref = source_ref
     )
 
@@ -109,15 +107,15 @@ def getDictUnpackingHelper():
         name       = helper_name,
         doc        = None,
         parameters = ParameterSpec(
-            name          = helper_name,
-            normal_args   = (),
-            list_star_arg = "args",
-            dict_star_arg = None,
-            default_count = 0,
-            kw_only_args  = ()
+            ps_name          = helper_name,
+            ps_normal_args   = (),
+            ps_list_star_arg = "args",
+            ps_dict_star_arg = None,
+            ps_default_count = 0,
+            ps_kw_only_args  = ()
         ),
-        source_ref = internal_source_ref,
-        is_class   = False
+        flags      = set(),
+        source_ref = internal_source_ref
     )
 
     temp_scope = None
@@ -143,22 +141,19 @@ def getDictUnpackingHelper():
                 source_ref   = internal_source_ref
             ),
             exception_name = "StopIteration",
-            handler_body   = StatementBreakLoop(
+            handler_body   = StatementLoopBreak(
                 source_ref = internal_source_ref
             ),
             source_ref     = internal_source_ref
         ),
         makeTryExceptSingleHandlerNode(
-            tried          = StatementExpressionOnly(
-                expression = ExpressionDictOperationUpdate(
-                    dict_arg   = ExpressionTempVariableRef(
-                        variable   = tmp_result_variable,
-                        source_ref = internal_source_ref
-                    ),
-                    value      = ExpressionTempVariableRef(
-                        variable   = tmp_item_variable,
-                        source_ref = internal_source_ref
-                    ),
+            tried          = StatementDictOperationUpdate(
+                dict_arg   = ExpressionTempVariableRef(
+                    variable   = tmp_result_variable,
+                    source_ref = internal_source_ref
+                ),
+                value      = ExpressionTempVariableRef(
+                    variable   = tmp_item_variable,
                     source_ref = internal_source_ref
                 ),
                 source_ref = internal_source_ref
@@ -170,7 +165,7 @@ def getDictUnpackingHelper():
                     args           = (
                         ExpressionOperationBinary(
                             operator   = "Mod",
-                            left       =  ExpressionConstantRef(
+                            left       =  makeConstantRefNode(
                                 constant      = """\
 '%s' object is not a mapping""",
                                 source_ref    = internal_source_ref,
@@ -246,7 +241,7 @@ def getDictUnpackingHelper():
                 variable   = tmp_result_variable,
                 source_ref = internal_source_ref
             ),
-            source       = ExpressionConstantRef(
+            source       = makeConstantRefNode(
                 constant   = {},
                 source_ref = internal_source_ref
             ),
@@ -294,16 +289,15 @@ def buildDictionaryUnpackingArgs(provider, keys, values, source_ref):
                 ExpressionMakeDict(
                     pairs      = (
                         ExpressionKeyValuePair(
-                            key        = ExpressionConstantRef(
-                                            constant   = key,
-                                            source_ref = source_ref
+                            key        = makeConstantRefNode(
+                                constant   = key,
+                                source_ref = source_ref
                             ),
                             value      = buildNode(provider, value, source_ref),
                             source_ref = source_ref
                         ),
                     ),
-                    source_ref = source_ref,
-                    lazy_order = False,
+                    source_ref = source_ref
                 )
             )
         else:
@@ -316,8 +310,7 @@ def buildDictionaryUnpackingArgs(provider, keys, values, source_ref):
                             source_ref = source_ref
                         ),
                     ),
-                    source_ref = source_ref,
-                    lazy_order = False,
+                    source_ref = source_ref
                 )
             )
 
